/******************************************
 * AppleIIGo Widget interface
 * Copyright (C) 2006, Marc S. Ressl.
 * All rights reserved.
 ******************************************/

var isRestart = false;

var displayColorMode;

var diskDrive1;
var diskDrive1Cache;
var diskDrive2;
var diskDrive2Cache;

/*****************************
 * Helpers
 *****************************/

function getWidgetDir() {
	var widgetDir = widget.system('/bin/pwd', null).outputString;
	return widgetDir.substring(0, widgetDir.length - 1);
}

function getFileList(folder) {
	var files = widget.system('/bin/ls ' + folder, null).outputString;
	if (files == undefined)
		files = '';
	files = files.split("\n");
	if (files.length > 0)
		files.length--;
	return files;
}

function loadRom() {
	// Load ROM
	var roms = getFileList(getWidgetDir() + '/ROMs');
	for(var i = 0; i < roms.length; i++) {
		if (appleiigo.loadRom(getWidgetDir() + '/ROMs/' + roms[i]))
			break;
	}
}

function updateMachine() {
	// Color mode
	appleiigo.setColorMode(displayColorMode);

	// Disks
	if (diskDrive1 == diskDrive2)
		diskDrive2 = '';

	// First unmount second drive, in case we re-mount it in the first drive
	if (diskDrive2 != diskDrive2Cache)
		appleiigo.mountDisk(1, getWidgetDir() + '/Disks/' + diskDrive2);
	if (diskDrive1 != diskDrive1Cache)
		appleiigo.mountDisk(0, getWidgetDir() + '/Disks/' + diskDrive1);

	diskDrive1Cache = diskDrive1;
	diskDrive2Cache = diskDrive2;

	// Restart
	if (isRestart) {
		isRestart = false;
		appleiigo.restart();
	}
}

/*****************************
 * Animation
 *****************************/

function limit_3(a, b, c) {
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat(from, to, ease) {
    return from + (to - from) * ease;
}

function animate(animation) {
	var T;
	var ease;
	var time = (new Date).getTime();

	T = limit_3(time - animation.starttime, 0, animation.duration);
	
	if (T >= animation.duration) {
		clearInterval(animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	} else {
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat(animation.from, animation.to, ease);
	}
	
	animation.element.style.opacity = animation.now;
}

/*****************************
 * Main functions
 *****************************/

function setup() {
	if(window.widget) {
		createGenericButton(document.getElementById('disks-control'), 'Disks', showDisks);
		createGenericButton(document.getElementById('roms-control'), 'ROMs', showRoms);
		createGenericButton(document.getElementById('restart-control'), 'Restart', restartMachine);
		createGenericButton(document.getElementById('done-control'), 'Done', hidePrefs);
	}

	if (window.widget) {
		window.onfocus = onfocus;
		window.onblur = onblur;
	}

	// Load configuration
	diskDrive1 = widget.preferenceForKey('diskDrive1');
	diskDrive2 = widget.preferenceForKey('diskDrive2');
	displayColorMode = widget.preferenceForKey('displayColorMode');

	if (diskDrive1 == undefined) {
		var disks = getFileList(getWidgetDir() + '/Disks');
		if (disks.length > 0)
			diskDrive1 = disks[0];
	}
	if (diskDrive2 == undefined)
		diskDrive2 = '';
	if (displayColorMode == undefined)
		displayColorMode = 1;

	// Set disk activity LED
	setInterval('updateDiskActivity();', 500);

	// Set configuration
	loadRom();
	updateMachine();

	// Resume
	appleiigo.resume();
}

function cleanup() {
}

function updateDisks() {
	var selectDrive1 = document.getElementById('drive-1-select');
	selectDrive1.options.length = 0;
	selectDrive1.options[0] = new Option('No disk', '');
	var selectDrive2 = document.getElementById('drive-2-select');
	selectDrive2.options.length = 0;
	selectDrive2.options[0] = new Option('No disk', '');

	var disks = getFileList(getWidgetDir() + '/Disks');
	for(var i = 0; i < disks.length; i++) {
		selectDrive1.options[i + 1] = new Option(disks[i]);
		if (diskDrive1 == disks[i])
			selectDrive1.options[i + 1].selected = true;
		selectDrive2.options[i + 1] = new Option(disks[i]);
		if (diskDrive2 == disks[i])
			selectDrive2.options[i + 1].selected = true;
	}
}

function showPrefs() {
	// Populate disks
	updateDisks();

	// Populate color mode
	var selectColorMode = document.getElementById('monitor-select');
	selectColorMode.options[displayColorMode].selected = true;

	// Flip
	var front = document.getElementById('front');
 	var back = document.getElementById('back');
	
	if (window.widget)
		widget.prepareForTransition('ToBack');
	
	front.style.opacity=0;
//	front.style.display='none';
	back.style.display='block';
	appleiigo.pause();
	appleiigo.hide();
	
	if (window.widget)
		setTimeout('widget.performTransition();', 0);

	document.getElementById('fliprollie').style.opacity = 0;
}

function hidePrefs() {
	// Apply configuration
	var selectDrive1 = document.getElementById('drive-1-select');
	diskDrive1 = selectDrive1.options[selectDrive1.selectedIndex].value;
	var selectDrive2 = document.getElementById('drive-2-select');
	diskDrive2 = selectDrive2.options[selectDrive2.selectedIndex].value;
	var selectColorMode = document.getElementById('monitor-select');
	displayColorMode = selectColorMode.options[selectColorMode.selectedIndex].value;

	widget.setPreferenceForKey(diskDrive1, 'diskDrive1');
	widget.setPreferenceForKey(diskDrive2, 'diskDrive2');
	widget.setPreferenceForKey(String(displayColorMode), 'displayColorMode');

	// Flip
	var front = document.getElementById('front');
	var back = document.getElementById('back');
	
	if (window.widget)
		widget.prepareForTransition('ToFront');
	
	appleiigo.show();
	appleiigo.resume();
	back.style.display='none';
	front.style.opacity=1;

	updateMachine();

	if (window.widget)
		setTimeout('widget.performTransition();appleiigo.focus()', 500);
}

function restartMachine() {
	isRestart = true;
	loadRom();
	hidePrefs();
}

/*****************************
 * Events
 *****************************/

function onfocus() {
	updateDisks();
	appleiigo.resume();
}

function onblur() {
	appleiigo.pause();
}

function enterFlip() {
	document.getElementById('fliprollie').style.opacity = 0.25;
}

function exitFlip() {
	document.getElementById('fliprollie').style.opacity = 0;
}

function updateDiskActivity() {
	if (appleiigo.getDiskActivity())
		document.getElementById('diskled-control').style.opacity = 1;
	else
		document.getElementById('diskled-control').style.opacity = 0;
}

var flipAnimation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, element:null, timer:null};
var flipShown = false;

function flipMouseOver() {
	if (!flipShown) {
		// fade in the flip widget
		if (flipAnimation.timer != null) {
			clearInterval(flipAnimation.timer);
			flipAnimation.timer = null;
		}
		
		var starttime = (new Date).getTime() - 13; // set it back one frame
		
		flipAnimation.duration = 500;
		flipAnimation.starttime = starttime;
		flipAnimation.element = document.getElementById('flip');
		flipAnimation.timer = setInterval('animate(flipAnimation);', 13);
		flipAnimation.from = flipAnimation.now;
		flipAnimation.to = 1.0;
		animate(flipAnimation);
		flipShown = true;
	}
}

function flipMouseOut() {
	if (flipShown) {
		if (flipAnimation.timer != null) {
			clearInterval(flipAnimation.timer);
			flipAnimation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13; // set it back one frame
		
		flipAnimation.duration = 500;
		flipAnimation.starttime = starttime;
		flipAnimation.element = document.getElementById('flip');
		flipAnimation.timer = setInterval('animate(flipAnimation);', 13);
		flipAnimation.from = flipAnimation.now;
		flipAnimation.to = 0.0;
		animate(flipAnimation);
		flipShown = false;
	}
}

function openFolder(folder) {
	var status = widget.system('/usr/bin/open ' + folder, null).status;
}

function showDisks() {
	openFolder(getWidgetDir() + '/Disks');
}

function showRoms() {
	openFolder(getWidgetDir() + '/ROMs');
}

function showInfo() {
	if (window.widget) widget.openURL('http://www-personal.umich.edu/~mressl/appleiigo');
}

